
#ifndef _nrf_H_
#define _nrf_H_
#define F_CPU 8000000UL

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

// user define

#define USE_IRQ_PIN //user irq pin
#define USE_INTERRUPT // use interrupt

//Architecture dependent function

#define CE  D,6 //ce piin nrf24l01p
#define CSN B,0 //csn piin nrf24l01p
#define IRQ C,0 //irq piin nrf24l01p if no use irq pin not define

#ifdef USE_INTERRUPT
#define NRF_INTERRUPT	ISR(PCINT1_vect) //vector you interrupt
#define INIT_INTERRUPT 	PCMSK1=1;PCIFR|=0x2;PCICR|=0x2; // config register to enable interrupt
#endif

// end user define



#define NRF_CSN_HI     SET(PORT,CSN);
#define NRF_CSN_LO     CLR(PORT,CSN);
#define NRF_CE_HI      SET(PORT,CE);
#define NRF_CE_LO      CLR(PORT,CE);


#ifdef USE_IRQ_PIN 
#define INITPORT 	SET(DDR,CSN);SET(DDR,CE); CLR(DDR,IRQ); SET(PORT,IRQ); 
#else 
#define INITPORT 	SET(DDR,CSN);SET(DDR,CE);
#endif
#define avr_spi_send SPDR = byte;	while((SPSR & (1<<SPIF))==0); return (SPDR);

// end Architecture dependent function



typedef uint8_t byte;
typedef uint16_t word;
typedef uint32_t dword;

#define CONFIG      0x00
#define EN_AA       0x01
#define EN_RXADDR   0x02
#define SETUP_AW    0x03
#define SETUP_RETR  0x04
#define RF_CH       0x05
#define RF_SETUP    0x06
#define STATUS      0x07
#define OBSERVE_TX  0x08
#define CD          0x09
#define RX_ADDR_P0  0x0A
#define RX_ADDR_P1  0x0B
#define RX_ADDR_P2  0x0C
#define RX_ADDR_P3  0x0D
#define RX_ADDR_P4  0x0E
#define RX_ADDR_P5  0x0F
#define TX_ADDR     0x10
#define RX_PW_P0    0x11
#define RX_PW_P1    0x12
#define RX_PW_P2    0x13
#define RX_PW_P3    0x14
#define RX_PW_P4    0x15
#define RX_PW_P5    0x16
#define FIFO_STATUS 0x17
#define DYNPD		0x1c
#define FEATURE		0x1d

/* Bit Mnemonics */
#define MASK_RX_DR  6
#define MASK_TX_DS  5
#define MASK_MAX_RT 4
#define EN_CRC      3
#define CRCO        2
#define PWR_UP      1
#define PRIM_RX     0

#define ENAA_P5     5
#define ENAA_P4     4
#define ENAA_P3     3
#define ENAA_P2     2
#define ENAA_P1     1
#define ENAA_P0     0

#define ERX_P5      5
#define ERX_P4      4
#define ERX_P3      3
#define ERX_P2      2
#define ERX_P1      1
#define ERX_P0      0

#define AW          0
#define ARD         4
#define ARC         0

#define PLL_LOCK    4
#define RF_DR       3
#define RF_PWR      1
#define LNA_HCURR   0

#define RX_DR       6
#define TX_DS       5
#define MAX_RT      4
#define RX_P_NO     1
#define TX_FULL     0

#define PLOS_CNT    4
#define ARC_CNT     0

#define TX_REUSE    6
#define FIFO_FULL   5
#define TX_EMPTY    4
#define RX_FULL     1
#define RX_EMPTY    0

#define DPL_P0      0
#define DPL_P1      1
#define DPL_P2      2
#define DPL_P3      3
#define DPL_P4      4
#define DPL_P5      5

#define EN_DPL		2
#define EN_ACK_PAY	1
#define EN_DYN_ACK	0


/* Instruction Mnemonics */
#define R_REGISTER    0x00
#define W_REGISTER    0x20
#define REGISTER_MASK 0x1F
#define R_RX_PAYLOAD  0x61
#define W_TX_PAYLOAD  0xA0
#define FLUSH_TX      0xE1
#define FLUSH_RX      0xE2
#define REUSE_TX_PL   0xE3
#define NRF_NOP       0xFF
#define R_RX_PL_WID	  0x60
#define ACTIVATE	  0x50
#define W_ACK_PAYLOAD 0xa8
#define W_TX_PAYLOAD_NOACK 0xb0


#ifdef USE_IRQ_PIN
#define NRF_CONFIG     0x0
#else
#define NRF_CONFIG     0x70
#endif

#define NRF_POWERDOWN NRF_Send_Cmd( CONFIG, NRF_CONFIG | ( (0<<PWR_UP) | (1<<PRIM_RX) | ((nrf_config.NRF_CRC_Enable?1:0)<<EN_CRC)|((nrf_config.NRF_CRC_Mode?1:0)<<CRCO) ))
#define NRF_TX_POWERUP NRF_Send_Cmd( CONFIG, NRF_CONFIG | ( (1<<PWR_UP) | (0<<PRIM_RX) | ((nrf_config.NRF_CRC_Enable?1:0)<<EN_CRC)|((nrf_config.NRF_CRC_Mode?1:0)<<CRCO) ))
#define NRF_RX_POWERUP NRF_Send_Cmd( CONFIG, NRF_CONFIG | ( (1<<PWR_UP) | (1<<PRIM_RX) | ((nrf_config.NRF_CRC_Enable?1:0)<<EN_CRC)|((nrf_config.NRF_CRC_Mode?1:0)<<CRCO) ))

typedef enum
{
	NRF24L01_FREE = 0,
	NRF24L01_RECV_PACKET = 0x01,
	NRF24L01_SEND_OK     = 0x02,
	NRF24L01_SEND_ERROR  = 0x04,
	NRF24L01_TRANSMIT_P  = 0x10,
	NRF24L01_UNKNOW_ERR  = 0x40,
	NRF24L01_CHIP_ERR    = 0x80
	
}NRF24L01_CurStatusTypeDef;

typedef enum
{
	NRF24L01_DATA_SEND_NO_ACK_RECEIVED=1,
	NRF24L01_DATA_SEND_ACK_RECEIVED_OK,
	NRF24L01_DATA_SEND_NOACK_OK ,
	NRF24L01_DATA_SEND_OK,
	NRF24L01_CHIP_ERROR=0x80,
	NRF24L01_NO_DATA_RECEIVED=0xfe,
	NRF24L01_TX_FULL=0xff
}NRF24L01_RET_STATUS;

typedef struct
{
	uint8_t NRF_CRC_Enable;				//CRC enable
	uint8_t NRF_CRC_Mode;				//CRC mode
	uint8_t NRF_Mode_RX;		  			//0 - powerdown, 1- RX
	uint8_t NRF_Pipe_Auto_Ack_0;			//0x01 EN_AA Enhanced ShockBurst
	uint8_t NRF_Pipe_Auto_Ack_1;			//0x01 EN_AA Enhanced ShockBurst
	uint8_t NRF_Pipe_Auto_Ack_2;			//0x01 EN_AA Enhanced ShockBurst
	uint8_t NRF_Pipe_Auto_Ack_3;			//0x01 EN_AA Enhanced ShockBurst
	uint8_t NRF_Pipe_Auto_Ack_4;			//0x01 EN_AA Enhanced ShockBurst
	uint8_t NRF_Pipe_Auto_Ack_5;			//0x01 EN_AA Enhanced ShockBurst
	uint8_t NRF_Enable_Pipe_0;			//Enabling PIPEx
	uint8_t NRF_Enable_Pipe_1;			//Enabling PIPEx
	uint8_t NRF_Enable_Pipe_2;			//Enabling PIPEx
	uint8_t NRF_Enable_Pipe_3;			//Enabling PIPEx
	uint8_t NRF_Enable_Pipe_4;			//Enabling PIPEx
	uint8_t NRF_Enable_Pipe_5;			//Enabling PIPEx
	uint8_t NRF_Adres_Length;			//3,4,5 bytes adress length
	uint8_t NRF_Channel;				//0...126 RF channel
	uint8_t NRF_TX_Power;				//0,-6,-12,-18 dB RF Power for TX
	uint8_t NRF_Data_Rate;				//0 - 250 kbit/s,1 - 1 Mbit/s, 2 - 2 Mbit/s
	uint8_t NRF_RX_Adress_Pipe0[5];		//Adress for PIPE0
	uint8_t NRF_RX_Adress_Pipe1[5];		//Adress for PIPE1
	uint8_t NRF_RX_Adress_Pipe2;		//Adress for PIPE2
	uint8_t NRF_RX_Adress_Pipe3;		//Adress for PIPE3
	uint8_t NRF_RX_Adress_Pipe4;		//Adress for PIPE4
	uint8_t NRF_RX_Adress_Pipe5;		//Adress for PIPE5
	uint8_t NRF_TX_Adress[5];			//Adress for Transfer
	uint8_t NRF_Payload_Size_Pipe0;		//0...32 bytes for PIPE0
	uint8_t NRF_Payload_Size_Pipe1;		//0...32 bytes for PIPE1
	uint8_t NRF_Payload_Size_Pipe2;		//0...32 bytes for PIPE2
	uint8_t NRF_Payload_Size_Pipe3;		//0...32 bytes for PIPE3
	uint8_t NRF_Payload_Size_Pipe4;		//0...32 bytes for PIPE4
	uint8_t NRF_Payload_Size_Pipe5;		//0...32 bytes for PIPE5
	uint8_t NRF_Auto_Retransmit_Delay;		//0...32 delay between retransmits
	uint8_t NRF_Auto_Retransmit_Count;		//0...32 retrasmit counts
	uint8_t NRF_En_No_Ack;				//Enable send payload no ack
	uint8_t	NRF_Status;					//get status
#ifndef USE_INTERRUPT
	uint16_t DS_Delay;                  //delay send packet
#endif
	NRF24L01_CurStatusTypeDef cur_status;					//cur status nrf (tx packet, rx mode, rx packet, power down mode)
	uint16_t send_ok_count;
	uint16_t send_nok_count;
	uint16_t recv_count;
	void *nrf_buf;
	void (*spi_init)();

}NRF_CONF_DEF;

NRF_CONF_DEF nrf_config;

extern uint8_t spi_send(uint8_t byte);
extern uint8_t ReadReg(uint8_t reg, uint8_t *recv, uint8_t size);
void WriteReg(uint8_t cmd, uint8_t *write, uint8_t size);
extern void nrf_init();
extern void nrf_configure();
extern void nrf_read_config();
extern NRF24L01_RET_STATUS nrf_send(uint8_t * value, uint8_t len,uint8_t noack);
extern uint8_t nrf_get_data(uint8_t *data) ;
extern uint8_t nrf_read_status();
extern void NRF_Send_Cmd(uint8_t reg, uint8_t value);
extern uint8_t NRF_Read_Cmd(uint8_t reg);
extern uint8_t nrf_read_retr_count();
extern uint8_t nrf_read_fifo_status();
void nrf_check ();

#define GLUE(a, b)     a##b
#define GLUE_(a, b, c)     a##b,c
#define SET_(what, p, m) GLUE(what, p) |= (1 << (m))
#define CLR_(what, p, m) GLUE(what, p) &= ~(1 << (m))
#define GET_(/* PIN, */ p, m) GLUE(PIN, p) & (1 << (m))
#define GETINV_(/* PIN, */ p, m) (GLUE(PIN, p) & (1 << (m))) ^ (1 << (m))
#define SET(what, x) SET_(what, x)
#define CLR(what, x) CLR_(what, x)
#define GET(/* PIN, */ x) GET_(x)
#define GETINV(/* PIN, */ x) GETINV_(x)

#endif /* _nrf_H_ */
